@@ -125,7 +125,10 @@ def test_operator() -> AbstractOperator:
             """
 
             for document in documents:
-                document["new_field"] = 3
+                if "new_field" not in document:
+                    document["new_field"] = 0
+
+                document["new_field"] += 3
 
             return documents
 
@@ -165,6 +168,7 @@ def test_sentiment_workflow_token(test_client: Client) -> str:
     yield workflow_token
     test_client.delete_dataset(dataset_id)
 
+
 @pytest.fixture(scope="function")
 def test_sentiment_workflow_document_token(test_client: Client) -> str:
     salt = "".join(random.choices(string.ascii_lowercase, k=10))
@@ -174,7 +178,7 @@ def test_sentiment_workflow_document_token(test_client: Client) -> str:
         job_id=job_id,
         authorizationToken=test_client._token,
         text_field="sample_1_label",
-        documents=mock_documents(10).to_json()
+        documents=mock_documents(10).to_json(),
     )
     config_string = json.dumps(config)
     config_bytes = config_string.encode()
