@@ -1,41 +1,62 @@
-import requests
 import time
 import uuid
 import logging
+import requests
 import traceback
+
 from json import JSONDecodeError
 from functools import wraps
+
 from typing import Any, Dict, List, Optional
+
 from workflows_core.utils import document
-from workflows_core.types import Credentials, FieldTransformer, Filter, Schema
+from workflows_core.types import (
+    Credentials,
+    FieldTransformer,
+    Filter,
+    Schema,
+    GroupBy,
+    Metric,
+)
+
 from workflows_core import __version__
 
 logger = logging.getLogger(__name__)
+logger.setLevel(logging.DEBUG)
 
 
 def get_response(response: requests.Response) -> Dict[str, Any]:
     # get a json response
     # if errors - print what the response contains
-    try:
-        return response.json()
-    except Exception as e:
-        logger.error({"error": e})
+    if response.status_code == 200:
         try:
-            # Log this somewhere if it errors
-            logger.error(response.content)
-        except Exception as no_content_e:
-            # in case there's no content
-            logger.error(no_content_e)
-        finally:
-            # we still want to raise the right error for retrying
-            # continue to raise exception so that any retry logic still holds
+            return response.json()
+        except Exception as e:
+            logger.error({"error": e, "x-trace-id": response.headers["x-trace-id"]})
             raise e
+    else:
+        if "x-trace-id" in response.headers:
+            logger.error(
+                {
+                    "x-trace-id": response.headers["x-trace-id"],
+                    "error": response.content,
+                }
+            )
+        else:
+            logger.error({"error": response.content})
+    try:
+        # Log this somewhere if it errors
+        logger.error(response.content)
+    except Exception as no_content_e:
+        # in case there's no content
+        logger.error(no_content_e)
+        # we still want to raise the right error for retrying
+        # continue to raise exception so that any retry logic still holds
+        raise no_content_e
 
 
 # We implement retry as a function for several reasons
 # first - we can get a
-
-
 def retry(num_of_retries: int = 3, timeout: int = 30):
     """
     Allows the function to retry upon failure.
@@ -272,6 +293,7 @@ class API:
         send_email: bool = True,
         worker_number: int = None,
         output: dict = None,
+        email: dict = None,
     ):
         # add edge case for API
         if job_id == "":
@@ -289,12 +311,17 @@ class API:
             additional_information=additional_information,
             send_email=send_email,
         )
-        if worker_number is None:
+        if worker_number is not None:
             parameters["worker_number"] = worker_number
 
         if output:
             parameters["output"] = {"results": output}
 
+        if email:
+            # in the form of
+            # 'secondary_cta': { 'url': <url>, 'text': <text>}
+            parameters["email"] = email
+
         response = requests.post(
             url=self._base_url + f"/workflows/{job_id}/status",
             headers=self._headers,
@@ -478,6 +505,9 @@ class API:
             ),
             workflow_id=workflow_id,
             version=version,
+            notebook_path="",
+            instance_type="batch",
+            host_type="batch",
         )
 
     @retry()
@@ -635,16 +665,41 @@ class API:
 
     @retry()
     def _update_keyphrase(
-        self, dataset_id: str, field: str, keyphrase_id: str, alias: str
+        self,
+        dataset_id: str,
+        field: str,
+        keyphrase_id: str,
+        alias: str,
+        keyphrase: str,
+        frequency: int = 0,
+        ancestors: list = None,
+        parents: list = None,
+        metadata: dict = None,
+        keyphrase_score: float = 0,
+        level: int = 0,
     ):
         # missing update contents here?
         """
         Update keyphrases
         """
+        params = {
+            "_id": keyphrase_id,
+            "text": keyphrase,
+            "frequency": frequency,
+            "keyphrase_score": keyphrase_score,
+            "level": level,
+        }
+        if ancestors is not None:
+            params["ancestors"] = ancestors
+        if parents is not None:
+            params["parents"] = parents
+        if metadata is not None:
+            params["metadata"] = metadata
         response = requests.post(
             url=self._base_url
             + f"/datasets/{dataset_id}/fields/{field}.{alias}/keyphrase/{keyphrase_id}/update",
             headers=self._headers,
+            json=params,
         )
         return get_response(response)
 
@@ -656,16 +711,22 @@ class API:
         alias: str,
         page: int = 0,
         page_size: int = 100,
-        sort: list = [],
+        sort: list = None,
     ):
         """
         List keyphrases
         """
+        params = {
+            "page": page,
+            "page_size": page_size,
+        }
+        if sort is not None:
+            params["sort"] = sort
         response = requests.post(
             url=self._base_url
             + f"/datasets/{dataset_id}/fields/{field}.{alias}/keyphrase/list",
             headers=self._headers,
-            json={"page": page, "page_size": page_size, "sort": sort},
+            json=params,
         )
         return get_response(response)
 
@@ -773,3 +834,72 @@ class API:
             ),
         )
         return get_response(response)
+
+    @retry()
+    def _label_openai(
+        self,
+        dataset_id: str,
+        vector_field: str,
+        field: str,
+        alias: str,
+        question_suffix: str,
+        accuracy: int = 4,
+        cluster_ids: list = None,
+        dont_save_summaries: bool = True,
+        filters: list = None,
+    ):
+        params = {
+            "vector_fields": [vector_field],
+            # legacy parameter
+            "centroid_vector_fields": [vector_field],
+            "alias": alias,
+            "dataset_id": dataset_id,
+            "cluster_ids": cluster_ids,
+            "dont_save_summaries": dont_save_summaries,
+            "questions": [
+                {
+                    "cluster_ids": cluster_ids,
+                    "config": {
+                        "accuracy": accuracy,
+                        "examples": [],
+                        "field": field,
+                        "question_suffix": question_suffix,
+                    },
+                }
+            ],
+        }
+        if filters is not None:
+            params["filters"] = filters
+        response = requests.get(
+            url=self._base_url
+            + f"/datasets/{dataset_id}/cluster/centroids/labels/create",
+            headers=self._headers,
+            params=params,
+        )
+        return get_response(response)
+
+    @retry()
+    def _aggregate(
+        self,
+        dataset_id: str,
+        page_size: str = 20,
+        page: str = 1,
+        asc: str = False,
+        aggregation_query: Dict[str, List[Dict[str, Any]]] = None,
+        dataset_ids: List[str] = None,
+        filters: List[Filter] = None,
+    ):
+        response = requests.post(
+            url=self._base_url + f"/datasets/{dataset_id}/aggregate",
+            headers=self._headers,
+            json=dict(
+                filters=[] if filters is None else filters,
+                aggregation_query=aggregation_query,
+                page_size=page_size,
+                page=page,
+                asc=asc,
+                dataset_ids=[] if dataset_ids is None else dataset_ids,
+                dataset_id=dataset_id,
+            ),
+        )
+        return get_response(response)
