@@ -1,7 +1,7 @@
 import logging
 
 from inspect import Traceback
-from typing import Any, Dict, List, Optional
+from typing import Any, Dict, List, Optional, Union
 
 from workflows_core.api.api import API
 from workflows_core.dataset.dataset import Dataset
@@ -27,7 +27,7 @@ class WorkflowContextManager(API):
         job_id: str,
         engine: AbstractEngine,
         dataset: Dataset,
-        operator: AbstractOperator,
+        operators: List[AbstractOperator],
         metadata: Optional[Dict[str, Any]] = None,
         additional_information: str = "",
         send_email: bool = True,
@@ -36,14 +36,21 @@ class WorkflowContextManager(API):
         super().__init__(dataset.api._credentials, job_id, workflow_name)
 
         self._engine = engine
-        self._operator = operator
         self._dataset = dataset
         self._dataset_id = dataset.dataset_id
 
-        self._update_field_children = (
-            self._operator._input_fields is not None
-            and self._operator._output_fields is not None
-        )
+        update_field_children = False
+        for operator in operators:
+            if (
+                operator._input_fields is not None
+                and operator._output_fields is not None
+            ):
+                update_field_children = True
+                break
+
+        self._operators = operators
+        self._update_field_children = update_field_children
+
         self._workflow_name = workflow_name
         self._job_id = job_id
 
@@ -92,13 +99,14 @@ class WorkflowContextManager(API):
             if self._mark_as_complete_after_polling:
                 # TODO: trigger a polling job while keeping this one in progress
                 # When triggering this poll job - we can send the job ID
-                self._trigger_polling_workflow(
+                result = self._trigger_polling_workflow(
                     dataset_id=self._dataset_id,
-                    input_field=self._operator._input_fields[0],
-                    output_field=self._operator._output_fields[0],
+                    input_field=self._operators[0]._input_fields[0],
+                    output_field=self._operators[-1]._output_fields[0],
                     job_id=self._job_id,
                     workflow_name=self._workflow_name,
                 )
+                logger.debug({"trigger_poll_id": result})
             else:
                 self._set_status(
                     status=self.COMPLETE,
@@ -106,15 +114,16 @@ class WorkflowContextManager(API):
                     output=self._engine.output_documents,
                 )
             if self._update_field_children:
-                for input_field in self._operator._input_fields:
-                    self._set_field_children(
-                        dataset_id=self._dataset_id,
-                        fieldchildren_id=self._workflow_name.lower().replace(
-                            "workflow", ""
-                        ),
-                        field=input_field,
-                        field_children=self._operator._output_fields,
-                    )
+                for operator in self._operators:
+                    for input_field in operator._input_fields:
+                        self._set_field_children(
+                            dataset_id=self._dataset_id,
+                            fieldchildren_id=self._workflow_name.lower().replace(
+                                "workflow", ""
+                            ),
+                            field=input_field,
+                            field_children=operator._output_fields,
+                        )
             return True
 
     def _set_status(
